from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.user_dto_login_type import UserDtoLoginType
from ..models.user_dto_user_deactivation_reason import UserDtoUserDeactivationReason
from qubicon.api.types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.privilege_dto import PrivilegeDto
    from ..models.role_dto import RoleDto
    from ..models.stream_dto import StreamDto


T = TypeVar("T", bound="UserDto")


@_attrs_define
class UserDto:
    """
    Attributes:
        name (str):
        first_name (str):
        last_name (str):
        email (str):
        roles (List['RoleDto']):
        id (Union[Unset, int]):
        activated (Union[Unset, bool]):
        gmp_enabled (Union[Unset, bool]):
        email_confirmed (Union[Unset, bool]):
        phone_sms_alerts (Union[Unset, str]):
        streams (Union[Unset, List['StreamDto']]):
        privileges (Union[Unset, List['PrivilegeDto']]):
        creation_date (Union[Unset, int]):
        update_date (Union[Unset, int]):
        failed_login_attempts (Union[Unset, int]):
        deactivation_time (Union[Unset, int]):
        activation_time (Union[Unset, int]):
        password_change_date (Union[Unset, int]):
        login_type (Union[Unset, UserDtoLoginType]):
        ldap_cn (Union[Unset, str]):
        user_deactivation_reason (Union[Unset, UserDtoUserDeactivationReason]):
        oauth_password_is_set (Union[Unset, bool]):
        oauth_id (Union[Unset, str]):
    """

    name: str
    first_name: str
    last_name: str
    email: str
    roles: List["RoleDto"]
    id: Union[Unset, int] = UNSET
    activated: Union[Unset, bool] = UNSET
    gmp_enabled: Union[Unset, bool] = UNSET
    email_confirmed: Union[Unset, bool] = UNSET
    phone_sms_alerts: Union[Unset, str] = UNSET
    streams: Union[Unset, List["StreamDto"]] = UNSET
    privileges: Union[Unset, List["PrivilegeDto"]] = UNSET
    creation_date: Union[Unset, int] = UNSET
    update_date: Union[Unset, int] = UNSET
    failed_login_attempts: Union[Unset, int] = UNSET
    deactivation_time: Union[Unset, int] = UNSET
    activation_time: Union[Unset, int] = UNSET
    password_change_date: Union[Unset, int] = UNSET
    login_type: Union[Unset, UserDtoLoginType] = UNSET
    ldap_cn: Union[Unset, str] = UNSET
    user_deactivation_reason: Union[Unset, UserDtoUserDeactivationReason] = UNSET
    oauth_password_is_set: Union[Unset, bool] = UNSET
    oauth_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        first_name = self.first_name

        last_name = self.last_name

        email = self.email

        roles = []
        for roles_item_data in self.roles:
            roles_item = roles_item_data.to_dict()
            roles.append(roles_item)

        id = self.id

        activated = self.activated

        gmp_enabled = self.gmp_enabled

        email_confirmed = self.email_confirmed

        phone_sms_alerts = self.phone_sms_alerts

        streams: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.streams, Unset):
            streams = []
            for streams_item_data in self.streams:
                streams_item = streams_item_data.to_dict()
                streams.append(streams_item)

        privileges: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.privileges, Unset):
            privileges = []
            for privileges_item_data in self.privileges:
                privileges_item = privileges_item_data.to_dict()
                privileges.append(privileges_item)

        creation_date = self.creation_date

        update_date = self.update_date

        failed_login_attempts = self.failed_login_attempts

        deactivation_time = self.deactivation_time

        activation_time = self.activation_time

        password_change_date = self.password_change_date

        login_type: Union[Unset, str] = UNSET
        if not isinstance(self.login_type, Unset):
            login_type = self.login_type.value

        ldap_cn = self.ldap_cn

        user_deactivation_reason: Union[Unset, str] = UNSET
        if not isinstance(self.user_deactivation_reason, Unset):
            user_deactivation_reason = self.user_deactivation_reason.value

        oauth_password_is_set = self.oauth_password_is_set

        oauth_id = self.oauth_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "firstName": first_name,
                "lastName": last_name,
                "email": email,
                "roles": roles,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if activated is not UNSET:
            field_dict["activated"] = activated
        if gmp_enabled is not UNSET:
            field_dict["gmpEnabled"] = gmp_enabled
        if email_confirmed is not UNSET:
            field_dict["emailConfirmed"] = email_confirmed
        if phone_sms_alerts is not UNSET:
            field_dict["phoneSmsAlerts"] = phone_sms_alerts
        if streams is not UNSET:
            field_dict["streams"] = streams
        if privileges is not UNSET:
            field_dict["privileges"] = privileges
        if creation_date is not UNSET:
            field_dict["creationDate"] = creation_date
        if update_date is not UNSET:
            field_dict["updateDate"] = update_date
        if failed_login_attempts is not UNSET:
            field_dict["failedLoginAttempts"] = failed_login_attempts
        if deactivation_time is not UNSET:
            field_dict["deactivationTime"] = deactivation_time
        if activation_time is not UNSET:
            field_dict["activationTime"] = activation_time
        if password_change_date is not UNSET:
            field_dict["passwordChangeDate"] = password_change_date
        if login_type is not UNSET:
            field_dict["loginType"] = login_type
        if ldap_cn is not UNSET:
            field_dict["ldapCn"] = ldap_cn
        if user_deactivation_reason is not UNSET:
            field_dict["userDeactivationReason"] = user_deactivation_reason
        if oauth_password_is_set is not UNSET:
            field_dict["oauthPasswordIsSet"] = oauth_password_is_set
        if oauth_id is not UNSET:
            field_dict["oauthId"] = oauth_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.privilege_dto import PrivilegeDto
        from ..models.role_dto import RoleDto
        from ..models.stream_dto import StreamDto

        d = src_dict.copy()
        name = d.pop("name")

        first_name = d.pop("firstName")

        last_name = d.pop("lastName")

        email = d.pop("email")

        roles = []
        _roles = d.pop("roles")
        for roles_item_data in _roles:
            roles_item = RoleDto.from_dict(roles_item_data)

            roles.append(roles_item)

        id = d.pop("id", UNSET)

        activated = d.pop("activated", UNSET)

        gmp_enabled = d.pop("gmpEnabled", UNSET)

        email_confirmed = d.pop("emailConfirmed", UNSET)

        phone_sms_alerts = d.pop("phoneSmsAlerts", UNSET)

        streams = []
        _streams = d.pop("streams", UNSET)
        for streams_item_data in _streams or []:
            streams_item = StreamDto.from_dict(streams_item_data)

            streams.append(streams_item)

        privileges = []
        _privileges = d.pop("privileges", UNSET)
        for privileges_item_data in _privileges or []:
            privileges_item = PrivilegeDto.from_dict(privileges_item_data)

            privileges.append(privileges_item)

        creation_date = d.pop("creationDate", UNSET)

        update_date = d.pop("updateDate", UNSET)

        failed_login_attempts = d.pop("failedLoginAttempts", UNSET)

        deactivation_time = d.pop("deactivationTime", UNSET)

        activation_time = d.pop("activationTime", UNSET)

        password_change_date = d.pop("passwordChangeDate", UNSET)

        _login_type = d.pop("loginType", UNSET)
        login_type: Union[Unset, UserDtoLoginType]
        if isinstance(_login_type, Unset):
            login_type = UNSET
        else:
            login_type = UserDtoLoginType(_login_type)

        ldap_cn = d.pop("ldapCn", UNSET)

        _user_deactivation_reason = d.pop("userDeactivationReason", UNSET)
        user_deactivation_reason: Union[Unset, UserDtoUserDeactivationReason]
        if isinstance(_user_deactivation_reason, Unset):
            user_deactivation_reason = UNSET
        else:
            user_deactivation_reason = UserDtoUserDeactivationReason(_user_deactivation_reason)

        oauth_password_is_set = d.pop("oauthPasswordIsSet", UNSET)

        oauth_id = d.pop("oauthId", UNSET)

        user_dto = cls(
            name=name,
            first_name=first_name,
            last_name=last_name,
            email=email,
            roles=roles,
            id=id,
            activated=activated,
            gmp_enabled=gmp_enabled,
            email_confirmed=email_confirmed,
            phone_sms_alerts=phone_sms_alerts,
            streams=streams,
            privileges=privileges,
            creation_date=creation_date,
            update_date=update_date,
            failed_login_attempts=failed_login_attempts,
            deactivation_time=deactivation_time,
            activation_time=activation_time,
            password_change_date=password_change_date,
            login_type=login_type,
            ldap_cn=ldap_cn,
            user_deactivation_reason=user_deactivation_reason,
            oauth_password_is_set=oauth_password_is_set,
            oauth_id=oauth_id,
        )

        user_dto.additional_properties = d
        return user_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class AbstractComputableModelDtoEngineType(str, Enum):
    EXTERNAL_PYTHON = "EXTERNAL_PYTHON"
    JYTHON = "JYTHON"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class OfflineEquipmentDtoStatus(str, Enum):
    AUTO_IMPORT = "AUTO_IMPORT"
    DEACTIVATED = "DEACTIVATED"
    DRAFT = "DRAFT"
    GMP = "GMP"
    READY = "READY"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.channel_data_channel_request_key_dto import ChannelDataChannelRequestKeyDto


T = TypeVar("T", bound="MultiplexChartDataChannelsRequestDto")


@_attrs_define
class MultiplexChartDataChannelsRequestDto:
    """
    Attributes:
        channels (List['ChannelDataChannelRequestKeyDto']):
    """

    channels: List["ChannelDataChannelRequestKeyDto"]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        channels = []
        for channels_item_data in self.channels:
            channels_item = channels_item_data.to_dict()
            channels.append(channels_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "channels": channels,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.channel_data_channel_request_key_dto import ChannelDataChannelRequestKeyDto

        d = src_dict.copy()
        channels = []
        _channels = d.pop("channels")
        for channels_item_data in _channels:
            channels_item = ChannelDataChannelRequestKeyDto.from_dict(channels_item_data)

            channels.append(channels_item)

        multiplex_chart_data_channels_request_dto = cls(
            channels=channels,
        )

        multiplex_chart_data_channels_request_dto.additional_properties = d
        return multiplex_chart_data_channels_request_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ProcessDtoProcessArchiveTransitionStatus(str, Enum):
    ARCHIVED = "ARCHIVED"
    ARCHIVE_IN_PROGRESS = "ARCHIVE_IN_PROGRESS"
    ARCHIVE_MANUALLY_SCHEDULED = "ARCHIVE_MANUALLY_SCHEDULED"
    UNARCHIVED_MANUALLY = "UNARCHIVED_MANUALLY"
    UNARCHIVE_IN_PROGRESS = "UNARCHIVE_IN_PROGRESS"
    UNTOUCHED = "UNTOUCHED"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class OnlineEquipmentNodeVariableDtoSignalType(str, Enum):
    CALIBRATION = "CALIBRATION"
    DATA = "DATA"
    STATE = "STATE"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from qubicon.api.types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.channel_data_dto import ChannelDataDto
    from ..models.channel_data_key_dto import ChannelDataKeyDto


T = TypeVar("T", bound="ChannelDataPairDto")


@_attrs_define
class ChannelDataPairDto:
    """
    Attributes:
        key (Union[Unset, ChannelDataKeyDto]):
        value (Union[Unset, List['ChannelDataDto']]):
    """

    key: Union[Unset, "ChannelDataKeyDto"] = UNSET
    value: Union[Unset, List["ChannelDataDto"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        key: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.key, Unset):
            key = self.key.to_dict()

        value: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.value, Unset):
            value = []
            for value_item_data in self.value:
                value_item = value_item_data.to_dict()
                value.append(value_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if key is not UNSET:
            field_dict["key"] = key
        if value is not UNSET:
            field_dict["value"] = value

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.channel_data_dto import ChannelDataDto
        from ..models.channel_data_key_dto import ChannelDataKeyDto

        d = src_dict.copy()
        _key = d.pop("key", UNSET)
        key: Union[Unset, ChannelDataKeyDto]
        if isinstance(_key, Unset):
            key = UNSET
        else:
            key = ChannelDataKeyDto.from_dict(_key)

        value = []
        _value = d.pop("value", UNSET)
        for value_item_data in _value or []:
            value_item = ChannelDataDto.from_dict(value_item_data)

            value.append(value_item)

        channel_data_pair_dto = cls(
            key=key,
            value=value,
        )

        channel_data_pair_dto.additional_properties = d
        return channel_data_pair_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Auto-import all public API functions from OpenAPI-generated files in public_api_controller_2"""

import pkgutil
import importlib

# Dynamically import all modules in this directory
__all__ = []

for module_info in pkgutil.iter_modules(__path__):
    module = importlib.import_module(f"{__name__}.{module_info.name}")
    globals()[module_info.name] = module
    __all__.append(module_info.name)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.abstract_computable_model_dto_calculation_style import AbstractComputableModelDtoCalculationStyle
from ..models.abstract_computable_model_dto_engine_type import AbstractComputableModelDtoEngineType
from ..models.abstract_computable_model_dto_status import AbstractComputableModelDtoStatus
from qubicon.api.types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.computable_model_input_dto import ComputableModelInputDto
    from ..models.computable_model_output_dto import ComputableModelOutputDto
    from ..models.sensor_type_dto import SensorTypeDto


T = TypeVar("T", bound="AbstractComputableModelDto")


@_attrs_define
class AbstractComputableModelDto:
    """
    Attributes:
        engine_type (AbstractComputableModelDtoEngineType):
        kpi_name (str):
        abbr (str):
        calculation_style (AbstractComputableModelDtoCalculationStyle):
        outputs (List['ComputableModelOutputDto']):
        id (Union[Unset, int]):
        sensor_type (Union[Unset, SensorTypeDto]):
        inputs (Union[Unset, List['ComputableModelInputDto']]):
        status (Union[Unset, AbstractComputableModelDtoStatus]):
        description (Union[Unset, str]):
        creation_date (Union[Unset, int]):
        update_date (Union[Unset, int]):
    """

    engine_type: AbstractComputableModelDtoEngineType
    kpi_name: str
    abbr: str
    calculation_style: AbstractComputableModelDtoCalculationStyle
    outputs: List["ComputableModelOutputDto"]
    id: Union[Unset, int] = UNSET
    sensor_type: Union[Unset, "SensorTypeDto"] = UNSET
    inputs: Union[Unset, List["ComputableModelInputDto"]] = UNSET
    status: Union[Unset, AbstractComputableModelDtoStatus] = UNSET
    description: Union[Unset, str] = UNSET
    creation_date: Union[Unset, int] = UNSET
    update_date: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        engine_type = self.engine_type.value

        kpi_name = self.kpi_name

        abbr = self.abbr

        calculation_style = self.calculation_style.value

        outputs = []
        for outputs_item_data in self.outputs:
            outputs_item = outputs_item_data.to_dict()
            outputs.append(outputs_item)

        id = self.id

        sensor_type: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.sensor_type, Unset):
            sensor_type = self.sensor_type.to_dict()

        inputs: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.inputs, Unset):
            inputs = []
            for inputs_item_data in self.inputs:
                inputs_item = inputs_item_data.to_dict()
                inputs.append(inputs_item)

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        description = self.description

        creation_date = self.creation_date

        update_date = self.update_date

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "engineType": engine_type,
                "kpiName": kpi_name,
                "abbr": abbr,
                "calculationStyle": calculation_style,
                "outputs": outputs,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if sensor_type is not UNSET:
            field_dict["sensorType"] = sensor_type
        if inputs is not UNSET:
            field_dict["inputs"] = inputs
        if status is not UNSET:
            field_dict["status"] = status
        if description is not UNSET:
            field_dict["description"] = description
        if creation_date is not UNSET:
            field_dict["creationDate"] = creation_date
        if update_date is not UNSET:
            field_dict["updateDate"] = update_date

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.computable_model_input_dto import ComputableModelInputDto
        from ..models.computable_model_output_dto import ComputableModelOutputDto
        from ..models.sensor_type_dto import SensorTypeDto

        d = src_dict.copy()
        engine_type = AbstractComputableModelDtoEngineType(d.pop("engineType"))

        kpi_name = d.pop("kpiName")

        abbr = d.pop("abbr")

        calculation_style = AbstractComputableModelDtoCalculationStyle(d.pop("calculationStyle"))

        outputs = []
        _outputs = d.pop("outputs")
        for outputs_item_data in _outputs:
            outputs_item = ComputableModelOutputDto.from_dict(outputs_item_data)

            outputs.append(outputs_item)

        id = d.pop("id", UNSET)

        _sensor_type = d.pop("sensorType", UNSET)
        sensor_type: Union[Unset, SensorTypeDto]
        if isinstance(_sensor_type, Unset):
            sensor_type = UNSET
        else:
            sensor_type = SensorTypeDto.from_dict(_sensor_type)

        inputs = []
        _inputs = d.pop("inputs", UNSET)
        for inputs_item_data in _inputs or []:
            inputs_item = ComputableModelInputDto.from_dict(inputs_item_data)

            inputs.append(inputs_item)

        _status = d.pop("status", UNSET)
        status: Union[Unset, AbstractComputableModelDtoStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = AbstractComputableModelDtoStatus(_status)

        description = d.pop("description", UNSET)

        creation_date = d.pop("creationDate", UNSET)

        update_date = d.pop("updateDate", UNSET)

        abstract_computable_model_dto = cls(
            engine_type=engine_type,
            kpi_name=kpi_name,
            abbr=abbr,
            calculation_style=calculation_style,
            outputs=outputs,
            id=id,
            sensor_type=sensor_type,
            inputs=inputs,
            status=status,
            description=description,
            creation_date=creation_date,
            update_date=update_date,
        )

        abstract_computable_model_dto.additional_properties = d
        return abstract_computable_model_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.role_dto_name import RoleDtoName
from qubicon.api.types import UNSET, Unset

T = TypeVar("T", bound="RoleDto")


@_attrs_define
class RoleDto:
    """
    Attributes:
        id (Union[Unset, int]):
        name (Union[Unset, RoleDtoName]):
    """

    id: Union[Unset, int] = UNSET
    name: Union[Unset, RoleDtoName] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        name: Union[Unset, str] = UNSET
        if not isinstance(self.name, Unset):
            name = self.name.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        _name = d.pop("name", UNSET)
        name: Union[Unset, RoleDtoName]
        if isinstance(_name, Unset):
            name = UNSET
        else:
            name = RoleDtoName(_name)

        role_dto = cls(
            id=id,
            name=name,
        )

        role_dto.additional_properties = d
        return role_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from setuptools import setup, find_packages


# Read dependencies from requirements.txt
def parse_requirements(filename):
    with open(filename, "r") as f:
        return [line.strip() for line in f if line.strip() and not line.startswith("#")]


setup(
    name="qubicon",
    version="3.5.14",
    author="Olivier Guillet",
    author_email="olivier.guillet@qubicon.io",
    description="Python SDK for interacting with the Qubicon platform through the library or CLI",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="",
    packages=find_packages(),
    install_requires=parse_requirements(
        "requirements.txt"
    ),  # Auto-updated dependencies
    entry_points={
        "console_scripts": [
            "qubicon-client=qubicon.cli:main",
        ],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.9",
)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.direct_to_process_import_file_progress_type import DirectToProcessImportFileProgressType
from qubicon.api.types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.xlsx_parsing_error_model import XlsxParsingErrorModel


T = TypeVar("T", bound="DirectToProcessImportFileProgress")


@_attrs_define
class DirectToProcessImportFileProgress:
    """
    Attributes:
        grand_failure (Union[Unset, bool]):
        grand_skipped (Union[Unset, bool]):
        errors (Union[Unset, List['XlsxParsingErrorModel']]):
        type (Union[Unset, DirectToProcessImportFileProgressType]):
        last_processed_record_number (Union[Unset, int]):
        total_rows (Union[Unset, int]):
        failed_rows (Union[Unset, int]):
        failed_cells_count (Union[Unset, int]):
        total_columns (Union[Unset, int]):
        existing_cells (Union[Unset, int]):
    """

    grand_failure: Union[Unset, bool] = UNSET
    grand_skipped: Union[Unset, bool] = UNSET
    errors: Union[Unset, List["XlsxParsingErrorModel"]] = UNSET
    type: Union[Unset, DirectToProcessImportFileProgressType] = UNSET
    last_processed_record_number: Union[Unset, int] = UNSET
    total_rows: Union[Unset, int] = UNSET
    failed_rows: Union[Unset, int] = UNSET
    failed_cells_count: Union[Unset, int] = UNSET
    total_columns: Union[Unset, int] = UNSET
    existing_cells: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        grand_failure = self.grand_failure

        grand_skipped = self.grand_skipped

        errors: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.errors, Unset):
            errors = []
            for errors_item_data in self.errors:
                errors_item = errors_item_data.to_dict()
                errors.append(errors_item)

        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        last_processed_record_number = self.last_processed_record_number

        total_rows = self.total_rows

        failed_rows = self.failed_rows

        failed_cells_count = self.failed_cells_count

        total_columns = self.total_columns

        existing_cells = self.existing_cells

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if grand_failure is not UNSET:
            field_dict["grandFailure"] = grand_failure
        if grand_skipped is not UNSET:
            field_dict["grandSkipped"] = grand_skipped
        if errors is not UNSET:
            field_dict["errors"] = errors
        if type is not UNSET:
            field_dict["type"] = type
        if last_processed_record_number is not UNSET:
            field_dict["lastProcessedRecordNumber"] = last_processed_record_number
        if total_rows is not UNSET:
            field_dict["totalRows"] = total_rows
        if failed_rows is not UNSET:
            field_dict["failedRows"] = failed_rows
        if failed_cells_count is not UNSET:
            field_dict["failedCellsCount"] = failed_cells_count
        if total_columns is not UNSET:
            field_dict["totalColumns"] = total_columns
        if existing_cells is not UNSET:
            field_dict["existingCells"] = existing_cells

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.xlsx_parsing_error_model import XlsxParsingErrorModel

        d = src_dict.copy()
        grand_failure = d.pop("grandFailure", UNSET)

        grand_skipped = d.pop("grandSkipped", UNSET)

        errors = []
        _errors = d.pop("errors", UNSET)
        for errors_item_data in _errors or []:
            errors_item = XlsxParsingErrorModel.from_dict(errors_item_data)

            errors.append(errors_item)

        _type = d.pop("type", UNSET)
        type: Union[Unset, DirectToProcessImportFileProgressType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = DirectToProcessImportFileProgressType(_type)

        last_processed_record_number = d.pop("lastProcessedRecordNumber", UNSET)

        total_rows = d.pop("totalRows", UNSET)

        failed_rows = d.pop("failedRows", UNSET)

        failed_cells_count = d.pop("failedCellsCount", UNSET)

        total_columns = d.pop("totalColumns", UNSET)

        existing_cells = d.pop("existingCells", UNSET)

        direct_to_process_import_file_progress = cls(
            grand_failure=grand_failure,
            grand_skipped=grand_skipped,
            errors=errors,
            type=type,
            last_processed_record_number=last_processed_record_number,
            total_rows=total_rows,
            failed_rows=failed_rows,
            failed_cells_count=failed_cells_count,
            total_columns=total_columns,
            existing_cells=existing_cells,
        )

        direct_to_process_import_file_progress.additional_properties = d
        return direct_to_process_import_file_progress

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class JobDtoStatus(str, Enum):
    CANCELATION_PENDING = "CANCELATION_PENDING"
    CANCELLED = "CANCELLED"
    COMPLETED = "COMPLETED"
    FAILED = "FAILED"
    IN_PROGRESS = "IN_PROGRESS"
    PLANNED = "PLANNED"

    def __str__(self) -> str:
        return str(self.value)



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from .. import errors
from ..client import AuthenticatedClient, Client
from qubicon.api.types import Response


def _get_kwargs(
    id: int,
) -> Dict[str, Any]:
    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": f"/public-api/jobs/{id}",
    }

    return _kwargs


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[Any]:
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    id: int,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Response[Any]:
    """
    Args:
        id (int):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        id=id,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    id: int,
    *,
    client: Union[AuthenticatedClient, Client],
) -> Response[Any]:
    """
    Args:
        id (int):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        id=id,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.process_phase_dto_status import ProcessPhaseDtoStatus
from ..models.process_phase_dto_type import ProcessPhaseDtoType
from qubicon.api.types import UNSET, Unset

T = TypeVar("T", bound="ProcessPhaseDto")


@_attrs_define
class ProcessPhaseDto:
    """
    Attributes:
        id (Union[Unset, int]):
        start_date (Union[Unset, int]):
        end_date (Union[Unset, int]):
        order (Union[Unset, int]):
        status (Union[Unset, ProcessPhaseDtoStatus]):
        name (Union[Unset, str]):
        description (Union[Unset, str]):
        type (Union[Unset, ProcessPhaseDtoType]):
        duration_min_ms (Union[Unset, int]):
        duration_max_ms (Union[Unset, int]):
        recipe_phase_id (Union[Unset, int]):
    """

    id: Union[Unset, int] = UNSET
    start_date: Union[Unset, int] = UNSET
    end_date: Union[Unset, int] = UNSET
    order: Union[Unset, int] = UNSET
    status: Union[Unset, ProcessPhaseDtoStatus] = UNSET
    name: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    type: Union[Unset, ProcessPhaseDtoType] = UNSET
    duration_min_ms: Union[Unset, int] = UNSET
    duration_max_ms: Union[Unset, int] = UNSET
    recipe_phase_id: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        start_date = self.start_date

        end_date = self.end_date

        order = self.order

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        name = self.name

        description = self.description

        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        duration_min_ms = self.duration_min_ms

        duration_max_ms = self.duration_max_ms

        recipe_phase_id = self.recipe_phase_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if start_date is not UNSET:
            field_dict["startDate"] = start_date
        if end_date is not UNSET:
            field_dict["endDate"] = end_date
        if order is not UNSET:
            field_dict["order"] = order
        if status is not UNSET:
            field_dict["status"] = status
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description
        if type is not UNSET:
            field_dict["type"] = type
        if duration_min_ms is not UNSET:
            field_dict["durationMinMs"] = duration_min_ms
        if duration_max_ms is not UNSET:
            field_dict["durationMaxMs"] = duration_max_ms
        if recipe_phase_id is not UNSET:
            field_dict["recipePhaseId"] = recipe_phase_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        start_date = d.pop("startDate", UNSET)

        end_date = d.pop("endDate", UNSET)

        order = d.pop("order", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, ProcessPhaseDtoStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = ProcessPhaseDtoStatus(_status)

        name = d.pop("name", UNSET)

        description = d.pop("description", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, ProcessPhaseDtoType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = ProcessPhaseDtoType(_type)

        duration_min_ms = d.pop("durationMinMs", UNSET)

        duration_max_ms = d.pop("durationMaxMs", UNSET)

        recipe_phase_id = d.pop("recipePhaseId", UNSET)

        process_phase_dto = cls(
            id=id,
            start_date=start_date,
            end_date=end_date,
            order=order,
            status=status,
            name=name,
            description=description,
            type=type,
            duration_min_ms=duration_min_ms,
            duration_max_ms=duration_max_ms,
            recipe_phase_id=recipe_phase_id,
        )

        process_phase_dto.additional_properties = d
        return process_phase_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from qubicon.api.types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.multi_language_label_dto import MultiLanguageLabelDto


T = TypeVar("T", bound="ErrorDto")


@_attrs_define
class ErrorDto:
    """
    Attributes:
        trace (Union[Unset, str]):
        message (Union[Unset, str]):
        multi_language_label (Union[Unset, MultiLanguageLabelDto]):
        reasons (Union[Unset, List['ErrorDto']]):
    """

    trace: Union[Unset, str] = UNSET
    message: Union[Unset, str] = UNSET
    multi_language_label: Union[Unset, "MultiLanguageLabelDto"] = UNSET
    reasons: Union[Unset, List["ErrorDto"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        trace = self.trace

        message = self.message

        multi_language_label: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.multi_language_label, Unset):
            multi_language_label = self.multi_language_label.to_dict()

        reasons: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.reasons, Unset):
            reasons = []
            for reasons_item_data in self.reasons:
                reasons_item = reasons_item_data.to_dict()
                reasons.append(reasons_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if trace is not UNSET:
            field_dict["trace"] = trace
        if message is not UNSET:
            field_dict["message"] = message
        if multi_language_label is not UNSET:
            field_dict["multiLanguageLabel"] = multi_language_label
        if reasons is not UNSET:
            field_dict["reasons"] = reasons

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.multi_language_label_dto import MultiLanguageLabelDto

        d = src_dict.copy()
        trace = d.pop("trace", UNSET)

        message = d.pop("message", UNSET)

        _multi_language_label = d.pop("multiLanguageLabel", UNSET)
        multi_language_label: Union[Unset, MultiLanguageLabelDto]
        if isinstance(_multi_language_label, Unset):
            multi_language_label = UNSET
        else:
            multi_language_label = MultiLanguageLabelDto.from_dict(_multi_language_label)

        reasons = []
        _reasons = d.pop("reasons", UNSET)
        for reasons_item_data in _reasons or []:
            reasons_item = ErrorDto.from_dict(reasons_item_data)

            reasons.append(reasons_item)

        error_dto = cls(
            trace=trace,
            message=message,
            multi_language_label=multi_language_label,
            reasons=reasons,
        )

        error_dto.additional_properties = d
        return error_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
